print("import: 'shapely'")
import shapely

print("import: 'shapely.algorithms'")
import shapely.algorithms

print("import: 'shapely.geometry'")
import shapely.geometry

print("import: 'shapely.vectorized'")
import shapely.vectorized

